#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.checker.interfaces import IContentChecker, ERROR_VALUE
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.form.interfaces import IWfForm, IForm, FORM_CONTENT_TYPE, FORM_CONTENT_NAME, \
    IFormFieldContainerTarget, IFormHandler, IFormFieldContainer

# import packages
from pyams_content.shared.common import WfSharedContent, SharedContent, register_content_type, WfSharedContentChecker
from pyams_utils.adapter import adapter_config
from zope.component.globalregistry import getGlobalSiteManager
from zope.interface import implementer, alsoProvides, noLongerProvides
from zope.schema.fieldproperty import FieldProperty

from pyams_content import _


@implementer(IWfForm, IFormFieldContainerTarget,
             IPreviewTarget, IReviewTarget)
class WfForm(WfSharedContent):
    """Base form"""

    content_type = FORM_CONTENT_TYPE
    content_name = FORM_CONTENT_NAME

    user_title = FieldProperty(IWfForm['user_title'])
    header = FieldProperty(IWfForm['header'])
    _handler = FieldProperty(IWfForm['handler'])
    auth_only = FieldProperty(IWfForm['auth_only'])
    use_captcha = FieldProperty(IWfForm['use_captcha'])
    submit_label = FieldProperty(IWfForm['submit_label'])

    @property
    def handler(self):
        return self._handler

    @handler.setter
    def handler(self, value):
        old_handler = self._handler
        if value == old_handler:
            return
        if old_handler is not None:
            handler = self.query_handler(old_handler)
            if (handler is not None) and handler.target_interface:
                noLongerProvides(self, handler.target_interface)
        if value is not None:
            handler = self.query_handler(value)
            if (handler is not None) and handler.target_interface:
                alsoProvides(self, handler.target_interface)
        self._handler = value

    def query_handler(self, handler=None):
        if handler is None:
            handler = self._handler
        if handler:
            registry = getGlobalSiteManager()
            return registry.queryUtility(IFormHandler, name=handler)

register_content_type(WfForm)


@implementer(IForm)
class Form(SharedContent):
    """Workflow managed form class"""

    content_class = WfForm


@adapter_config(name='properties', context=IWfForm, provides=IContentChecker)
class WfFormContentChecker(WfSharedContentChecker):
    """Form content checker"""

    def inner_check(self, request):
        output = super(WfFormContentChecker, self).inner_check(request)
        translate = request.localizer.translate
        if not IFormFieldContainer(self.context).get_fields():
            output.append(translate(ERROR_VALUE).format(field=translate(_("Form fields")),
                                                        message=translate(_("no field defined"))))
        return output
