#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.features.checker.interfaces import IContentChecker, ERROR_VALUE, MISSING_VALUE, MISSING_LANG_VALUE
from pyams_content.shared.imagemap.interfaces import IImageMapParagraph
from pyams_i18n.interfaces import II18nManager, INegotiator, II18n
from pyams_workflow.interfaces import IWorkflow

# import packages
from pyams_content.component.paragraph import BaseParagraph, BaseParagraphContentChecker
from pyams_sequence.utility import get_reference_target
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config, get_utility
from pyams_utils.traversing import get_parent
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content import _


@implementer(IImageMapParagraph)
class ImageMapParagraph(BaseParagraph):
    """Image map paragraph"""

    icon_class = 'fa-location-arrow'
    icon_hint = _("Image map")

    reference = FieldProperty(IImageMapParagraph['reference'])

    def get_target(self, state=None):
        return get_reference_target(self.reference, state)


@utility_config(name='ImageMap', provides=IParagraphFactory)
class ImageMapParagraphFactory(object):
    """Image map paragraph factory"""

    name = _("Image map")
    content_type = ImageMapParagraph


@adapter_config(context=IImageMapParagraph, provides=IContentChecker)
class ImageMapParagraphContentChecker(BaseParagraphContentChecker):
    """ImageMap paragraph content checker"""

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            value = i18n.get_attribute('title', lang, request)
            if not value:
                field_title = translate(IImageMapParagraph['title'].title)
                if len(langs) == 1:
                    output.append(translate(MISSING_VALUE).format(field=field_title))
                else:
                    output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        target = self.context.get_target()
        if target is not None:
            workflow = IWorkflow(target, None)
            if workflow is not None:
                output.append(translate(ERROR_VALUE).format(field=translate(IImageMapParagraph['reference'].title),
                                                            message=translate(_("target is not published"))))
        return output
