#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import ISiteFolder, ISiteManager, ISiteFolderRoles
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_i18n.interfaces import II18n
from pyams_portal.interfaces import IPortalContext
from pyams_security.interfaces import IDefaultProtectionPolicy
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.intid.interfaces import IIntIds

# import packages
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_content.shared.site.container import SiteContainerMixin
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.container import find_objects_providing
from pyams_utils.registry import get_local_registry
from pyams_utils.request import query_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


@implementer(IDefaultProtectionPolicy, ISiteFolder, ISiteFolderRoles,
             IPortalContext, IAttributeAnnotatable)
class SiteFolder(SiteContainerMixin, OrderedContainer, BaseSharedTool):
    """Site folder persistent class"""

    roles_interface = ISiteFolderRoles

    notepad = FieldProperty(ISiteFolder['notepad'])

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    def is_deletable(self):
        for element in self.values():
            if not element.is_deletable():
                return False
        return True


@adapter_config(context=ISiteFolder, provides=IFormContextPermissionChecker)
class SiteFolderPermissionChecker(ContextAdapter):
    """Site folder edit permission checker"""

    edit_permission = MANAGE_SITE_PERMISSION


@vocabulary_config(name='PyAMS site folders')
class SiteManagerFoldersVocabulary(SimpleVocabulary):
    """Site manager folders vocabulary"""

    def __init__(self, context):
        terms = []
        site = get_parent(context, ISiteManager)
        if site is not None:
            registry = get_local_registry()
            if registry is not None:
                request = query_request()
                intids = registry.getUtility(IIntIds)
                for folder in find_objects_providing(site, ISiteFolder):
                    terms.append(SimpleTerm(value=intids.queryId(folder),
                                            title=II18n(folder).query_attribute('title', request=request)))
        super(SiteManagerFoldersVocabulary, self).__init__(terms)
